/*

Wrapper to communicate with UI::WebDialog. 
Required by WebDialogX.rb

Version:      1.1
Date:         21.07.2012

*/

Beta = {};
Beta.UI = {};
Beta.UI.WebDialogX = {


  /* Wrapper to call any defined action_callback, handles escaping and encoding etc.
   * name, *arguments   or   {name: string, data: json, success: function, error: function, complete: function}
   */
  callback: function () {

    // don't continue if we are testing outside of SketchUp
    if(!/SketchUp/i.test(navigator.userAgent)) return;

    // merge the given arguments with
    var opt = {data: null, direct: false, indirect: true, success: null, error: null, complete: null},
      name = null,
      args = [];
    if(arguments.length == 1 && typeof(arguments[0]) === 'object'){
      for(i in arguments[0]){opt[i] = arguments[0][i]}
      name = opt.name;
      args.push(opt.data);
    } else if(arguments.length >= 1) {
      name = arguments[0];
      args = Array.prototype.slice.call(arguments).slice(1);
    } else {return false}

    var id = (this._messageIdCounter++);
    this._callbacks[id] = {success: opt.success, error: opt.error, complete: opt.complete};

    /* direct message passing
     *   encode message into url
     *   Ruby gets the message immediately
     */
    if (opt.direct == true ||  opt.indirect == false){
      var data = JSON.stringify( args );
      data = encodeURIComponent(data.replace(/%/g, "%25").replace(/'/g, "%q"));
      // split the argument string into chunks and encode each
      var chunks = data.match(/.{1,2060}(?=%|$)/g); // actually 2083 is maximum url size in Internet Explorer
      for(var i=0; i<chunks.length; i++){
        this._setLocation("skp:_collect_data@[" + id + "]" + chunks[i]);
      }
      this._setLocation("skp:" + name + "@" + id);
    }
    /* indirect message passing (preferred)
     *   put message into hidden text field
     *   tell Ruby about it
     *   Ruby gets the text field value
     */
    else if(opt.indirect == true || opt.direct == false){
      this._createMessageField(id, JSON.stringify(args));
      this._setLocation("skp:" + name + "@" + id);
    }
    return true; // the message has been sent
  },


  /* Call a Ruby method (even if a specific action_callback has not been defined)
   * Arguments: name and any amount of arguments  or  options object
   */
  callTo: function () {
    if(arguments.length == 1 && typeof(arguments[0]) === 'object'){
      var opt = arguments[0];
      opt.data = [opt.name, opt.data];
      opt.name = "_callTo";
      this.callback(opt);
    } else if(arguments.length >= 1) {
      var name_and_args = Array.prototype.slice.call(arguments).slice();
      name_and_args.unshift("_callTo");
      this.callback.apply(this, name_and_args);
    } else {return false}
  },


  /* Execute any Ruby code (even if a specific action_callback has not been defined)
   */
  execScript: function (code_string, options) {
    if(typeof(code_string) !== 'string'){ return false }
    if(typeof(options) === 'object'){
      options.data = code_string;
      options.name = "_execScript";
      return this.callback(options);
    } else {
      return this.callback("_execScript", code_string);
    }
  },


  // Keeps track of callbacks.
  _messageIdCounter: 0,
  _callbacks: {},


  // Utility methods.
  debug: false,
  _createMessageField: function (id, value) {
    var messageField = document.createElement('input');
    messageField.setAttribute('type', 'hidden');
    messageField.setAttribute('style', 'display:none');
    messageField.setAttribute('id', 'Beta.UI.WebDialogX.' + id);
    messageField.value = value;
    document.body.appendChild(messageField);
  },
  _setLocation: function (url) {
    // allow the DOM to refresh so that the MessageField is created (from DynamicComponents dcBridge.js)
    window.setTimeout(function() {
      window.location.href = url;
    }, 0);
  },
  _cleanUp: function (id) {
    try {
      var elem = document.getElementById('Beta.UI.WebDialogX.' + id);
      elem.parentNode.removeChild(elem);
    } catch(e) {
      if(this.debug == true){ this.callTo("puts", "Error in Beta.UI.WebDialogX._cleanUp:\\n" + e) };
    } finally {
      delete this._callbacks[id];
    };
  },
  _cleanUpScripts: function () {
    // timeout to wait after injected script has been executed (otherwise this one is not removed)
    window.setTimeout(function(){
      var scripts = document.body.getElementsByTagName("script");
      for(var i = 0; i < scripts.length; i++){ scripts[i].parentNode.removeChild(scripts[i]) };
    }, 0);
  },
  _fromRuby: function (id, functionName, argumentsString) {
    var returnValue = null;
    try{
      //Create the function
      var fn = eval(functionName); // alternative only for global functions: window[functionName]
      if(fn == null){ throw "function is not defined" };
      argumentsString = argumentsString;
      var data = eval(argumentsString);
      returnValue = fn.apply(fn, data);
    } catch(e){
      if(this.debug == true){ this.callTo("puts", "Error in Beta.UI.WebDialogX._fromRuby:\\n" + e) };
    } finally {
      this._createMessageField(id, JSON.stringify(returnValue));
    }
  },
  _execScript: function (data) {
    var returnValue = null;
    try{
      returnValue = eval(data);
    } catch(e){
      if(this.debug == true){ this.callTo("puts", "Error in Beta.UI.WebDialogX._execScript:\\n" + e) };
    };
    return returnValue;
  }
}
